@@ -1,11 +1,12 @@
 import logging
 from abc import abstractmethod, ABCMeta
-from collections import defaultdict, namedtuple
-from typing import Union, Optional
+from collections import namedtuple
+from typing import Union, Optional, List
 
 from twisted.internet import reactor
 from twisted.internet.defer import fail, Deferred, succeed, inlineCallbacks
 from twisted.python import failure
+from twisted.python.deprecate import deprecated
 
 from vortex.DeferUtil import vortexLogFailure
 from vortex.PayloadEndpoint import PayloadEndpoint
@@ -17,6 +18,20 @@ from vortex.VortexFactory import VortexFactory
 logger = logging.getLogger(__name__)
 
 
+class TupleSelectorUpdateMapperABC(metaclass=ABCMeta):
+    @abstractmethod
+    def mapTupleSelector(self, triggerTupleSelector: TupleSelector,
+                         allTupleSelectors: List[TupleSelector]) -> List[TupleSelector]:
+        """ Map Tuple Selector
+
+        Implementing this class provides some intelligence to the Observable
+        to trigger a refresh of related data to the triggerTupleSelector.
+
+        :param triggerTupleSelector: The TupleSelector that has triggered the update.
+        :param allTupleSelectors: All TupleSelectors being observed from this observable.
+        """
+
+
 class TuplesProviderABC(metaclass=ABCMeta):
     @abstractmethod
     def makeVortexMsg(self, filt: dict,
@@ -34,6 +49,12 @@ class TuplesProviderABC(metaclass=ABCMeta):
 _ObserverDetails = namedtuple("_ObserverDetails", ["vortexUuid", "observerName"])
 
 
+class _ObserverData:
+    def __init__(self, tupleSelector: TupleSelector):
+        self.tupleSelector = tupleSelector
+        self.observers = set()
+
+
 class TupleDataObservableHandler:
     def __init__(self, observableName,
                  additionalFilt=None,
@@ -61,9 +82,10 @@ class TupleDataObservableHandler:
         self._endpoint = PayloadEndpoint(self._filt, self._process,
                                          acceptOnlyFromVortex=acceptOnlyFromVortex)
 
-        self._observerDetailsByTupleSelector = defaultdict(set)
+        self._observerDataByTupleSelector = {}
 
         self._tupleProvidersByTupleName = {}
+        self._tupleSelectorUpdateMappers = []
 
     def addTupleProvider(self, tupleName, provider: TuplesProviderABC):
         """ Add Tuple Provider
@@ -79,14 +101,25 @@ class TupleDataObservableHandler:
 
         self._tupleProvidersByTupleName[tupleName] = provider
 
+    def addTupleSelectorUpdateMapper(self, mapper: TupleSelectorUpdateMapperABC):
+        """ Add Tuple Selector Update Mapper
+
+        This mapper will be called every time a tuple selector is notified of an update.
+
+        """
+        self._tupleSelectorUpdateMappers.append(mapper)
+
     def hasTupleProvider(self, tupleName: str):
         return tupleName in self._tupleProvidersByTupleName
 
-    def hasTupleSubscribers(self, tupleSelector:TupleSelector):
-        return tupleSelector.toJsonStr() in self._observerDetailsByTupleSelector
+    def hasTupleSubscribers(self, tupleSelector: TupleSelector):
+        return tupleSelector.toJsonStr() in self._observerDataByTupleSelector
 
     def shutdown(self):
         self._endpoint.shutdown()
+        self._tupleSelectorUpdateMappers = []
+        self._tupleProvidersByTupleName = {}
+        self._observerDataByTupleSelector = {}
 
     def _createVortexMsg(self, filt, tupleSelector: TupleSelector) -> Deferred:
         tupleProvider = self._tupleProvidersByTupleName.get(tupleSelector.name)
@@ -110,19 +143,25 @@ class TupleDataObservableHandler:
 
         # Handle unsubscribe
         if payloadEnvelope.filt.get("unsubscribe"):
-            try:
-                self._observerDetailsByTupleSelector[tsStr].remove(observerDetails)
-            except KeyError:
-                pass
+            if tsStr in self._observerDataByTupleSelector:
+                try:
+                    self._observerDataByTupleSelector[tsStr] \
+                        .observers.remove(observerDetails)
 
-            if not self._observerDetailsByTupleSelector[tsStr]:
-                del self._observerDetailsByTupleSelector[tsStr]
+                except KeyError:
+                    pass
+
+                if not self._observerDataByTupleSelector[tsStr].observers:
+                    del self._observerDataByTupleSelector[tsStr]
 
             return
 
         # Add support for just getting data, no subscription.
         if self._subscriptionsEnabled and payloadEnvelope.filt.get("subscribe", True):
-            self._observerDetailsByTupleSelector[tsStr].add(observerDetails)
+            if tsStr not in self._observerDataByTupleSelector:
+                self._observerDataByTupleSelector[tsStr] = _ObserverData(tupleSelector)
+
+            self._observerDataByTupleSelector[tsStr].observers.add(observerDetails)
 
         vortexMsg = yield self._createVortexMsg(payloadEnvelope.filt, tupleSelector)
         try:
@@ -130,6 +169,25 @@ class TupleDataObservableHandler:
         except Exception as e:
             logger.exception(e)
 
+    def _getMappedTupleSelectors(self, tupleSelector: TupleSelector
+                                 ) -> List[TupleSelector]:
+
+        # Get all tuple selectors
+        allTupleSelectors = [data.tupleSelector
+                             for data in self._observerDataByTupleSelector.values()]
+
+        # Create a dict so we end up with only unique ones
+        tupleSelectorByStr = {tupleSelector.toJsonStr(): tupleSelector}
+
+        # Run through the mappers
+        for mapper in self._tupleSelectorUpdateMappers:
+            mappedSelectors = mapper.mapTupleSelector(tupleSelector, allTupleSelectors)
+            if mappedSelectors:
+                tupleSelectorByStr.update({ts.toJsonStr(): ts for ts in mappedSelectors})
+
+        # Return a list of tuple selectors
+        return list(tupleSelectorByStr.values())
+
     def notifyOfTupleUpdate(self, tupleSelector: TupleSelector) -> None:
         """ Notify Of Tuple Update
 
@@ -142,8 +200,10 @@ class TupleDataObservableHandler:
         :returns: None
 
         """
-        reactor.callLater(0, self._notifyOfTupleUpdateInMain, tupleSelector)
+        tupleSelectors = self._getMappedTupleSelectors(tupleSelector)
+        reactor.callLater(0, self._notifyOfTupleUpdateInMain, tupleSelectors)
 
+    @deprecated("1.3.0")
     def notifyOfTupleUpdateForTuple(self, tupleName: str) -> None:
         """ Notify of Tuple Update for Tuple
 
@@ -151,26 +211,34 @@ class TupleDataObservableHandler:
         send it to observers.
 
         """
-        for tupleSelectorStr in self._observerDetailsByTupleSelector:
-            tupleSelector = TupleSelector.fromJsonStr(tupleSelectorStr)
+        tupleSelectors = []
+        for data in self._observerDataByTupleSelector.values():
+            if data.tupleSelector.name == tupleName:
+                tupleSelectors.append(data.tupleSelector)
 
-            if tupleSelector.name == tupleName:
-                self.notifyOfTupleUpdate(tupleSelector)
+        reactor.callLater(0, self._notifyOfTupleUpdateInMain, tupleSelectors)
 
     @inlineCallbacks
-    def _notifyOfTupleUpdateInMain(self, tupleSelector: TupleSelector):
+    def _notifyOfTupleUpdateInMain(self, tupleSelectors: List[TupleSelector]):
+        for tupleSelector in tupleSelectors:
+            yield self._notifyOfTupleUpdateInMainOne(tupleSelector)
+
+    @inlineCallbacks
+    def _notifyOfTupleUpdateInMainOne(self, tupleSelector: TupleSelector):
         tsStr = tupleSelector.toJsonStr()
+        if tsStr not in self._observerDataByTupleSelector:
+            return
 
         # Filter out the offline observables
         onlineUuids = set(VortexFactory.getRemoteVortexUuids())
-        observerDetails = self._observerDetailsByTupleSelector[tsStr]
-        for od in list(observerDetails):
+        observerDetails = self._observerDataByTupleSelector[tsStr].observers
+        for od in observerDetails:
             if od.vortexUuid not in onlineUuids:
                 observerDetails.remove(od)
 
         # Get / update the list of observing UUIDs
         if not observerDetails:
-            del self._observerDetailsByTupleSelector[tsStr]
+            del self._observerDataByTupleSelector[tsStr]
             return
 
         # Create the vortexMsg
